import numpy as np
import ctypes
from .lowlevel import _load_native_lib
from .types import GstflowBuffer, GstflowBufferLayout

def gstflow_process_image(image: np.ndarray) -> list[np.ndarray]:
    try: 
        # Load library
        lib = _load_native_lib()
        lib.gstflow_custom_process.argtypes = [ctypes.POINTER(GstflowBuffer), ctypes.POINTER(GstflowBuffer)]
        lib.gstflow_custom_process.restype = None
        lib.gstflow_free_buffer.argtypes = [ctypes.POINTER(GstflowBuffer)]
        lib.gstflow_free_buffer.restype = None

        h, w, c = image.shape
        input_buf = GstflowBuffer()
        input_buf.data = image.ctypes.data_as(ctypes.POINTER(ctypes.c_ubyte))
        input_buf.name = b"python-input"
        input_buf.w = w
        input_buf.h = h
        input_buf.c = c
        input_buf.n = 1
        input_buf.layout = GstflowBufferLayout.NHWC

        output_buf = GstflowBuffer()
        lib.gstflow_custom_process(ctypes.byref(input_buf), ctypes.byref(output_buf))

        frames = []
        size = output_buf.w * output_buf.h * output_buf.c
        data = np.ctypeslib.as_array(output_buf.data, shape=(output_buf.n * size,))

        for i in range(output_buf.n):
            start = i * size
            raw = data[start:start + size]

            if output_buf.layout == GstflowBufferLayout.NHWC:
                frame = raw.reshape((output_buf.h, output_buf.w, output_buf.c))
            elif output_buf.layout == GstflowBufferLayout.NCHW:
                frame = raw.reshape((output_buf.c, output_buf.h, output_buf.w)).transpose(1, 2, 0)
            else:
                raise ValueError(f"Unsupported output layout: {output_buf.layout}")
            frames.append(frame.copy())
        lib.gstflow_free_buffer(ctypes.byref(input_buf))
        lib.gstflow_free_buffer(ctypes.byref(output_buf))
        return frames

    except Exception as e:
        print("Error in gstflow_process_image:", e)
        return []
